package mx.redhat.samples.jdg.module;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.lucene.search.Query;
import org.hibernate.search.query.dsl.QueryBuilder;
import org.infinispan.Cache;
import org.infinispan.query.CacheQuery;
import org.infinispan.query.ResultIterator;
import org.infinispan.query.Search;
import org.infinispan.query.SearchManager;

import com.redhat.mx.coppel.afore.model.Resumen;
import com.redhat.mx.coppel.afore.model.Saldo;

/**
 * Sample application code.  For more examples visit http://community.jboss.org/wiki/5minutetutorialonInfinispan
 */
public class Application 
{
	public void load(Long load) 
	{
		System.out.println("\n\n1.  Demonstrating basic usage of Red Hat JBoss Data Grid.  This cache stores arbitrary objects.");
		Cache<String, Resumen> cache = SampleCacheContainer.getCache("resumen-cache");

        Date start = new Date();
        
        System.out.println("Loading cache with sample data.");
        
		long x = (load == null || load <= 0) ? 1000000 : load;
		
		NumberFormat nf = DecimalFormat.getIntegerInstance();
		nf.setMinimumIntegerDigits(8);
		nf.setMaximumFractionDigits(0);
		nf.setGroupingUsed(false);
		
		Resumen resumen;
        for (long i = 0; i < x; i++) 
        {
			resumen = new Resumen();
			resumen.setNss(nf.format(i+1));
			
			Saldo saldo = new Saldo();
			saldo.setValorInicial(Math.random()*1000);
			saldo.setValorFinal(Math.random()*10000);
			
			resumen.setSaldo(saldo);
			
			cache.put(resumen.getNss(), resumen);
        }

        Date l = new Date();
        
        System.out.println("Loaded cache with " + x + " entries in " + (l.getTime() - start.getTime()) + "ms");
        
		SearchManager manager = Search.getSearchManager(cache);
		
		manager.getMassIndexer().start();
		
		Date indx = new Date();
		
		System.out.println("Seconds to index " + x + " entries in " + (indx.getTime() - l.getTime()) + "ms");		
	}

	public void search() 
	{
		System.out.println("\n\n1.  Demonstrating search feature.");
		Cache<String, Resumen> cache = SampleCacheContainer.getCache("resumen-cache");

		Date jdg = new Date();
		
		SearchManager manager = Search.getSearchManager(cache);
		
		QueryBuilder builder = manager.buildQueryBuilderForClass(Resumen.class).get();
		Query luceneQuery = builder
				.range()
				.onField("saldo.valorInicial")
				.above(900.0)
				.createQuery();
		
		CacheQuery query = manager.getQuery(luceneQuery);
		
		List<Resumen> resumen = new ArrayList<Resumen>();
		
		ResultIterator miIt = query.iterator();
		while (miIt.hasNext())
		{
			Resumen item = (Resumen)miIt.next();
			resumen.add(item);
			System.out.println("nss: " + item.getNss() + " [" + item.getSaldo().getValorInicial() + "]");
		}
		
		System.out.printf("result size: " + resumen.size());

		Date qry = new Date();
		
		System.out.printf("Seconds to search qry: " + (qry.getTime() - jdg.getTime()) + "ms");
	}

	public static void main(String[] args) throws Exception {
		System.out.println("\n\n\n   ********************************  \n\n\n");
		System.out.println("Hello.  This is a sample application making use of Infinispan.");
		Application a = new Application();
		a.load(100l);
		a.search();
		System.out.println("Sample complete.");
		System.out.println("\n\n\n   ********************************  \n\n\n");
	}

}

